@@ -12,6 +12,7 @@ import six
 from acme import messages
 from acme.magic_typing import Optional  # pylint: disable=unused-import, no-name-in-module
 
+from certbot import compat
 from certbot import constants
 from certbot import errors
 from certbot import util
@@ -85,6 +86,7 @@ class PostArgParseSetupTest(test_util.ConfigTestCase):
         self.memory_handler.close()
         self.stream_handler.close()
         self.temp_handler.close()
+        self.devnull.close()
         super(PostArgParseSetupTest, self).tearDown()
 
     def test_common(self):
@@ -259,7 +261,7 @@ class TempHandlerTest(unittest.TestCase):
 
     def test_permissions(self):
         self.assertTrue(
-            util.check_permissions(self.handler.path, 0o600, os.getuid()))
+            util.check_permissions(self.handler.path, 0o600, compat.os_geteuid()))
 
     def test_delete(self):
         self.handler.close()
